Stan Math Library: stan/math/prim/scal/prob/rayleigh_cdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

rayleigh_cdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_RAYLEIGH_CDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_RAYLEIGH_CDF_LOG_HPP
3
4 #include <boost/random/uniform_real_distribution.hpp>
5 #include <boost/random/variate_generator.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive.hpp>
11 #include <stan/math/prim/scal/fun/log1m.hpp>
12 #include <stan/math/prim/scal/fun/square.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/meta/length.hpp>
15 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
16 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
17 #include <stan/math/prim/scal/fun/constants.hpp>
18 #include <stan/math/prim/scal/meta/include_summand.hpp>
19 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
20 #include <cmath>
21
22 namespace stan {
23
24 namespace math {
25
26 template <typename T_y, typename T_scale>
27 typename return_type<T_y, T_scale>::type
28 rayleigh_cdf_log(const T_y& y, const T_scale& sigma) {
29 static const char* function("stan::math::rayleigh_cdf_log");
30 typedef typename stan::partials_return_type<T_y, T_scale>::type
31 T_partials_return;
32
33 using stan::math::check_nonnegative;
34 using stan::math::check_positive;
35 using stan::math::check_not_nan;
36 using stan::math::check_consistent_sizes;
37 using stan::math::include_summand;
38 using stan::is_constant_struct;
39 using stan::math::square;
40 using stan::math::value_of;
41 using stan::math::log1m;
42 using std::exp;
43
44 T_partials_return cdf_log(0.0);
45
46 // check if any vectors are zero length
47 if (!(stan::length(y) && stan::length(sigma)))
48 return cdf_log;
49
50 check_not_nan(function, "Random variable", y);
51 check_nonnegative(function, "Random variable", y);
52 check_not_nan(function, "Scale parameter", sigma);
53 check_positive(function, "Scale parameter", sigma);
54 check_consistent_sizes(function,
55 "Random variable", y,
56 "Scale parameter", sigma);
57
58 // set up template expressions wrapping scalars into vector views
59 OperandsAndPartials<T_y, T_scale> operands_and_partials(y, sigma);
60
61 VectorView<const T_y> y_vec(y);
62 VectorView<const T_scale> sigma_vec(sigma);
63 size_t N = max_size(y, sigma);
64
65 VectorBuilder<true, T_partials_return, T_scale> inv_sigma(length(sigma));
66 for (size_t i = 0; i < length(sigma); i++) {
67 inv_sigma[i] = 1.0 / value_of(sigma_vec[i]);
68 }
69
70 for (size_t n = 0; n < N; n++) {
71 const T_partials_return y_dbl = value_of(y_vec[n]);
72 const T_partials_return y_sqr = y_dbl * y_dbl;
73 const T_partials_return inv_sigma_sqr = inv_sigma[n] * inv_sigma[n];
74 const T_partials_return exp_val = exp(-0.5 * y_sqr * inv_sigma_sqr);
75 const T_partials_return exp_div_1m_exp = exp_val / (1.0 - exp_val);
76
77 if (include_summand<false, T_y, T_scale>::value)
78 cdf_log += log1m(exp_val);
79
80 if (!is_constant_struct<T_y>::value)
81 operands_and_partials.d_x1[n] += y_dbl * inv_sigma_sqr
82 * exp_div_1m_exp;
83 if (!is_constant_struct<T_scale>::value)
84 operands_and_partials.d_x2[n] -= y_sqr * inv_sigma_sqr
85 * inv_sigma[n] * exp_div_1m_exp;
86 }
87
88 return operands_and_partials.value(cdf_log);
89 }
90 }
91 }
92 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
square.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::math::square
fvar< T > square(const fvar< T > &x)
Definition: square.hpp:15
include_summand.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::check_positive
bool check_positive(const char *function, const char *name, const T_y &y)
Return true if y is positive.
Definition: check_positive.hpp:68
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
log1m.hpp
stan::math::rayleigh_cdf_log
return_type< T_y, T_scale >::type rayleigh_cdf_log(const T_y &y, const T_scale &sigma)
Definition: rayleigh_cdf_log.hpp:28
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::log1m
fvar< T > log1m(const fvar< T > &x)
Definition: log1m.hpp:16
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.